/*///////////////////////////////////////////////////////////////////////
//                          SOURCE SAVE HEADER                         //
/////////////////////////////////////////////////////////////////////////
// Revision #    : $Revision:: 2                    $                  //
// Last change by: $Author:: Rkp                    $                  //
// Date & Time   : $Date:: 4/20/00 5:34p            $                  //
///////////////////////////////////////////////////////////////////////*/

#define RTN_OK     0
#define RTN_ERROR 13
#include <windows.h>
#include <stdio.h>
#include <tchar.h>

#define __USERHIVE_EXPORTS
#include "UserRegHive.h"

/////////////////////////////////////////////////////////////////////////
// Declarations
/////////////////////////////////////////////////////////////////////////

BOOL ObtainProfilePath( LPTSTR  pszSid, 
                        LPTSTR  pszProfilePath,
                        DWORD   dwPathSize );

BOOL Privilege(         LPTSTR  pszPrivilege, 
                        BOOL    bEnable );

BOOL Hive(              LPTSTR  pszSid, 
                        LPTSTR  pszProfilePath, 
                        BOOL    bLoad );


BOOL ObtainSid(         LPCTSTR szUser, 
                        LPTSTR  pszSid, 
                        LPDWORD pdwBufferLen );

BOOL LoadHive(          LPCTSTR lpszSid, 
                        BOOL    bLoad );

BOOL GetUserFromPath(   LPCTSTR lpszProfPath, 
                        LPTSTR  lpszFoundUser );

/////////////////////////////////////////////////////////////////////////
// Implemenatations
/////////////////////////////////////////////////////////////////////////

BOOL ObtainProfilePath( LPTSTR pszSid, 
                        LPTSTR pszProfilePath,
                        DWORD dwPathSize )
{
    TCHAR         szRegKey[1024] = TEXT( "SOFTWARE\\Microsoft\\" );
    TCHAR         szTemp[256] = TEXT( "" );
    DWORD         dwSizeProfilePath = dwPathSize;
    DWORD         dwType;
    HKEY          hKey;      
    LONG          lErrorCode;
    
    //
    // concat sid
    //
    lstrcat( szRegKey, TEXT( "Windows NT\\CurrentVersion\\ProfileList\\" ));
    lstrcat( szRegKey, pszSid );
    
    //      
    // find the hive in the registry
    //
    
    lErrorCode = RegOpenKeyEx( HKEY_LOCAL_MACHINE, szRegKey, 0, KEY_READ, &hKey );
    if ( lErrorCode != ERROR_SUCCESS )
        return FALSE;
    
    //
    // query the value
    //
    lErrorCode = RegQueryValueEx( hKey, TEXT( "ProfileImagePath" ), NULL, &dwType, ( LPBYTE ) pszProfilePath, &dwSizeProfilePath );
    if ( lErrorCode != ERROR_SUCCESS ) 
        return FALSE;
    
    //
    // fix profile path by replacing "%SystemRoot%" with the actual path
    //
    lstrcpy( szTemp, pszProfilePath );
    ZeroMemory( pszProfilePath, dwPathSize );
    if ( !ExpandEnvironmentStrings( szTemp, pszProfilePath,dwSizeProfilePath) )
        return FALSE;
    
    //
    // determine if it is a file or directory, WinNT 4.0 it is a
    // directory
    // WinNT 3.51 it is a file
    //
    if (( GetFileAttributes( pszProfilePath ) & FILE_ATTRIBUTE_DIRECTORY ) == FILE_ATTRIBUTE_DIRECTORY )
        lstrcat( pszProfilePath, TEXT( "\\ntuser.dat" ));
    
    //
    // close the key
    //
    lErrorCode = RegCloseKey( hKey );
    if ( lErrorCode != ERROR_SUCCESS )
        return FALSE;

    return TRUE;
}

BOOL Privilege( LPTSTR pszPrivilege, 
                BOOL bEnable )
{
    HANDLE           hToken;
    TOKEN_PRIVILEGES tp;
    
    //
    // obtain the processes token
    //
    if ( !OpenProcessToken( GetCurrentProcess(), TOKEN_ADJUST_PRIVILEGES | TOKEN_QUERY, &hToken ) )
        return FALSE;
    
    //      
    // get the luid
    //
    if ( !LookupPrivilegeValue( NULL, pszPrivilege, &tp.Privileges[0].Luid ) )
        return FALSE;
    
    tp.PrivilegeCount = 1;
    if ( bEnable )
        tp.Privileges[0].Attributes = SE_PRIVILEGE_ENABLED;
    else
        tp.Privileges[0].Attributes = 0;
    //
    // enable or disable the privilege
    //
    if ( !AdjustTokenPrivileges( hToken, FALSE, &tp, 0, ( PTOKEN_PRIVILEGES ) NULL, 0 ))
        return FALSE;

    if ( !CloseHandle( hToken ) )
        return FALSE;
    
    return TRUE;
}

BOOL Hive( LPTSTR pszSid, 
           LPTSTR pszProfilePath, 
           BOOL bLoad )
{
    LONG lErrorCode;
    if ( bLoad )
    {
        lErrorCode = RegLoadKey( HKEY_USERS, pszSid, pszProfilePath );
        if ( lErrorCode != ERROR_SUCCESS )
            return FALSE;
    }
    else
    {
        lErrorCode = RegUnLoadKey( HKEY_USERS, pszSid );
        if ( lErrorCode != ERROR_SUCCESS )
            return FALSE;
    }
    
    return TRUE;
}

BOOL GetUserFromPath( LPCTSTR lpszProfPath, 
                      LPTSTR  lpszFoundUser )
{
    LONG pos1 = 0;
    LONG pos2 = 0;

    //
    // The user is the last part of path...
    //
    while ( lpszProfPath[ pos1 ] != TEXT('\0') )
    {
        //
        // Look for backslashes and remember the position of the last one found.
        //
        if ( lpszProfPath[ pos1 ] == TEXT('\\') )
        {
            pos2 = pos1;
        }
        pos1 ++;
    }

    //
    // pos2 points to the last backslash...
    // copy everything from pos2 until the end to szFoundUser...
    //
    _tcscpy( lpszFoundUser, lpszProfPath + (( pos2 + 1 ) * sizeof( TCHAR )) );

    return TRUE;
}

BOOL ObtainSid( LPCTSTR lpszUser, 
                LPTSTR  lpszSid, 
                LPDWORD pdwBufferLen )
{
    HKEY hKey = NULL;
    HKEY hKeySid = NULL;
    
    if ( RegOpenKeyEx( HKEY_LOCAL_MACHINE, REG_WIN_USERS, 0, KEY_READ, &hKey ) == ERROR_SUCCESS )
    {
        //
        // Loop through all the subkeys...
        //
        LONG        ret = ERROR_SUCCESS;

        DWORD       dwSidLength = MAX_KEY_NAME_LENGTH;
        TCHAR       szFoundSid[ MAX_KEY_NAME_LENGTH + 1 ];

        DWORD       dwProfPathLength = MAX_KEY_VALUE_LENGTH;
        TCHAR       szProfPath[ MAX_KEY_VALUE_LENGTH + 1 ];

        DWORD       dwType;
        FILETIME    ft;

        TCHAR       szFoundUser[ MAX_KEY_VALUE_LENGTH + 1 ];

        for( int i = 0; ret == ERROR_SUCCESS; i++ )
        {
            dwSidLength = MAX_KEY_NAME_LENGTH;
            ret = RegEnumKeyEx( hKey, i, szFoundSid, &dwSidLength, NULL, NULL, NULL, &ft );
          
            //
            // Did we find a key ??
            //
            if ( ret == ERROR_SUCCESS )
            {
                // Yes.
                //
                // Open the SID key...
                //
                if ( RegOpenKeyEx( hKey, szFoundSid, 0, KEY_READ, &hKeySid ) == ERROR_SUCCESS )
                {
                    //
                    // Get the ProfileImagePath for this Sid...
                    //
                    dwProfPathLength = MAX_KEY_VALUE_LENGTH;
                    ret = RegQueryValueEx( hKeySid, REG_PROF_PATH, NULL, &dwType, (LPBYTE) szProfPath, &dwProfPathLength );

                    if ( ret == ERROR_SUCCESS )
                    {
                        //
                        // Parse the Path and see if the last part corresponds to the user that we are looking for...
                        //
                        GetUserFromPath( szProfPath, szFoundUser );

                        //
                        // Is this the right user??
                        //
                        if ( _tcsicmp( lpszUser, szFoundUser ) == 0 )
                        {
                            // Yes.
                            // Copy the found SID into the lpszSid parameter.
                            //
                            if ( (( LPDWORD ) ( _tcsclen( szFoundSid ) )) > pdwBufferLen )
                            {
                                RegCloseKey( hKeySid );
                                RegCloseKey( hKey );
                                return FALSE;
                            }
                            _tcscpy( lpszSid, szFoundSid );
                            RegCloseKey( hKeySid );
                            RegCloseKey( hKey );
                            return TRUE;
                        }
                    }
                    //
                    // Close the SID key...
                    //
                    RegCloseKey( hKeySid );
                    hKeySid = NULL;
                }
            }
        }

        RegCloseKey( hKey );
        return TRUE;
    }

    return FALSE;

}

IS_HIVE_API LONG WINAPI LoadHiveForUser( LPCTSTR szUser, 
                                         LPTSTR lpszSid, 
                                         LONG dwSidBufLen )
{
    TCHAR               szProfilePath[256];
    TCHAR               szSid[256];
    DWORD               dwBufferLen;
    
    ZeroMemory( szSid, ( sizeof( szSid ) / sizeof( TCHAR )));
    ZeroMemory( szProfilePath, ( sizeof( szProfilePath ) / sizeof( TCHAR )));
    dwBufferLen = ( sizeof( szSid ) / sizeof( TCHAR ));
    
    //
    // obtain the sid
    //
    if ( !ObtainSid( szUser, szSid, &dwBufferLen ) )
        return FALSE;

    //
    // check if the buffer is large enough
    //
    if ( (( LONG ) ( _tcsclen( szSid ) + 1 )) > dwSidBufLen )
        return FALSE;

    //
    // load the hive for the found SID.
    //
    if ( !LoadHive( szSid, TRUE ) )
        return FALSE;

    _tcscpy( lpszSid, szSid );

    return TRUE;
}

IS_HIVE_API LONG WINAPI LoadHiveForSid( LPCTSTR lpszSid )
{
    return LoadHive( lpszSid, TRUE );
}

IS_HIVE_API LONG WINAPI UnLoadHiveForSid( LPCTSTR lpszSid )
{
    return LoadHive( lpszSid, FALSE );
}

BOOL LoadHive( LPCTSTR lpszSid, 
               BOOL bLoad )
{
    TCHAR               szProfilePath[256];
    TCHAR               szSid[256];
    DWORD               dwBufferLen;
        
    //
    // initialize buffers
    //
    ZeroMemory( szSid, ( sizeof( szSid ) / sizeof( TCHAR )));
    ZeroMemory( szProfilePath, ( sizeof( szProfilePath ) / sizeof( TCHAR )));
    dwBufferLen = ( sizeof( szSid ) / sizeof( TCHAR ));
    
    //
    // check if a Sid has been submitted...
    //
    if (( lpszSid == NULL ) ||
        ( _tcsclen( lpszSid ) == 0 ))
        return FALSE;

    _tcscpy( szSid, lpszSid );
    
    //
    // obtain profile path
    //
    if ( !ObtainProfilePath( szSid, szProfilePath, ( sizeof( szProfilePath ) / sizeof( TCHAR ))))
        return FALSE;
    
    //
    // enable privilege
    //
    if ( !Privilege( SE_RESTORE_NAME, TRUE ) )
        return FALSE;
    
    //
    // load the hive
    //
    if ( !Hive( szSid, szProfilePath, bLoad ) )
        return FALSE;
    
    //
    // disable the privilege 
    //
    Privilege(SE_RESTORE_NAME, FALSE );
        
    return TRUE;
}
